MfgAide = {}
MfgAide.isMisn = false --true if goods were loaded for the current mission; used to unload goods at end of mission
MfgAide.isWarned = false --true after warning has been shown at least once each misn
MfgAide.ship_cargo = {} --cargo of active ship when mission started: compare to cargo after to unload new
MfgAide.version = '1.2'

--## Choco ##--

MfgAide.ui, MfgAide.vars = dofile'ui.lua'

local PRINT_COLOR = '\127ffffff'

--print message to chat with correct color
local function print_msg(msg)
	if not type(msg)=="String" then return false end
	print(PRINT_COLOR..msg)
end

--prints warning message if applicable, returns true if warning was shown
--if warn_override is true then show warning regardless of IsOneWarning vars setting
local function warning_msg(msg, warn_override)
	if not type(msg)=="String" then return false end
	
	if not MfgAide.isWarned or warn_override or not MfgAide.vars.GetVars"IsOneWarning" then
		print_msg('\127ff0000MfgAide WARNING\127o -- '..msg)
		MfgAide.isWarned = true --only show warning once per mission
		
		return true
	else return false end
end

local function is_mfg_misn()
	local mission_name = GetActiveMissionInfo(1) or ''
	return string.match(mission_name, "^Manufacture") or string.match(mission_name, "^Construct") --Mission name must begin with Manufacture or Construct
end

--Returns table of items to deliver for the active mission
local function get_items()
	local info = {GetActiveMissionInfo(1)}
	
	if info[1] then
		local mission_name = info[1]
		
		local needed_items = {}
		for _,obj in ipairs(info[4] or {}) do
			local num_reqd, item_name, sector, num_delivered = string.match((obj or {})[1] or "", "Deliver (%d+)x? (.+) to (%P+ %u%-%d+)%s*%[?(%d*)/?%d*%]?")
			local needed = (tonumber(num_reqd) or 0) - (tonumber(num_delivered) or 0) --remaining qty needed
			
			if needed>0 then				
				if not needed_items[item_name] then
					needed_items[item_name]=needed
				else needed_items[item_name] = needed_items[item_name] + needed end --combine multiple entries
			end
		end
		
		return needed_items
	end
	
	return false --no mission data available
end

--Compares inventory at current station to mission needed items and prints warning listing missing items
local function check_stock(options)
	local warn_override = (options or {})["warn_override"] or not MfgAide.vars.GetVars"IsOneWarning" --always show warning if parameter is set, otherwise use vars setting
	
	local needed_items = get_items() --qty of items needed by misn, item name as key
	local available_items = {} --qty of items needed from inventory, item name as key
	local purchaseable_items = {} --list of item names available to buy, item name as key
	
	if not needed_items then print_msg'You must have an active manufacturing mission to use MfgAide'; return end
	if not next(needed_items) then --[[print_msg'This mission has no items to deliver';]] return end --Mission completes too fast to get list of deliverable items now -- don't display message
	
	local stationid = GetStationLocation()
	if stationid then
		local item_list = {} --items in inventory needed by misn
		
		local sta_items = GetStationCargoList(stationid) --inventory at current station
		for _,itemid in ipairs(sta_items) do
			local name = GetInventoryItemName(itemid)
			local qty = GetInventoryItemQuantity(itemid) or 0
			local vol = GetInventoryItemVolume(itemid) or 0
			
			if needed_items[name] then
				local item = {itemid=itemid, name=name, qty=math.min(qty, needed_items[name]), vol=vol}
				table.insert(item_list, item)
				
				available_items[name] = (available_items[name] or 0) + qty
				
				needed_items[name] = math.max(needed_items[name] - qty, 0) --reduce needed qty by inventory qty
				if needed_items[name]==0 then needed_items[name] = nil end --remove item from list if all qty in inventory
			end
		end
		
		table.sort(item_list, function(a,b) return tonumber(a.vol)>tonumber(b.vol) end) --sort item_list, largest volume first
		
		--create list of items sold by station
		for n = 1, GetNumStationMerch() do
			purchaseable_items[GetStationMerchInfo(n).name] = true
		end
		
		--identify any needed items that are not available
		local is_missing = false --true if player doesn't have at least one required item
		local missing_items = 'The following items are not available in inventory: '
		for name,num in pairs(needed_items) do
			local color = "" --to insert color code into message string
			local end_color = "" --end of color code
			
			if purchaseable_items[name] then --color purchaseable items yellow
				color = "\127FFFF00"
				end_color = "\127o"
			end
			
			missing_items = missing_items.." "..color..num.."x "..name..end_color..","
			is_missing = true
		end
		
		--warning for not all items available
		if is_missing then warning_msg(missing_items, warn_override) end
		
		return true
	else print_msg'You must be docked at a station to use MfgAide'; return end
end

--checks misn active, sector and auto-loading enabled then loads items
local function tryAutoload()
	if is_mfg_misn() then
		if MfgAide.vars.GetVars"IsAutoUnload" and HasActiveShip() then
			MfgAide.ship_cargo = {}
			
			for _,itemid in ipairs( GetShipInventory(GetActiveShipID()).cargo ) do
				MfgAide.ship_cargo[itemid] = GetInventoryItemQuantity(itemid) or 0
			end
		end
	
		MfgAide.isMisn = true
		check_stock()
	end
end

function MfgAide:PLAYER_ENTERED_GAME(event, data)
	RegisterEvent(MfgAide, "SHOW_STATION")
	RegisterEvent(MfgAide, "HUD_SHOW")
end

function MfgAide:SHOW_STATION(event, data)
	UnregisterEvent(MfgAide, "SHOW_STATION")
	UnregisterEvent(MfgAide, "HUD_SHOW")
	RegisterEvent(MfgAide, "ENTERED_STATION")
	
	tryAutoload()
end

function MfgAide:HUD_SHOW(event, data)
	UnregisterEvent(MfgAide, "SHOW_STATION")
	UnregisterEvent(MfgAide, "HUD_SHOW")
	RegisterEvent(MfgAide, "ENTERED_STATION")
end

function MfgAide:MISSION_ADDED(event, data)
	RegisterEvent(MfgAide, "MISSION_UPDATED") --wait until mission updates to continue
end

--Check that all required items are available when mission begins
function MfgAide:MISSION_UPDATED(event, data)
	UnregisterEvent(MfgAide, "MISSION_UPDATED")
	
	tryAutoload()
end

--Unload all cargo when misn completes
function MfgAide:MISSION_REMOVED(event, data)	
	if MfgAide.isMisn then --only do unload if known to be mfg misn
		if MfgAide.vars.GetVars"IsAutoUnload" and HasActiveShip() then --do auto-unload
			local unload_items = {} --list of items to unload
			
			local ship_cargo = GetShipInventory(GetActiveShipID()).cargo
			for _,itemid in ipairs(ship_cargo) do
				local qty = (GetInventoryItemQuantity(itemid) or 0) - (MfgAide.ship_cargo[itemid] or 0) --find cargo that was not present earlier
				
				if qty>0 then table.insert(unload_items, {itemid=itemid, quantity=qty}) end
			end
			CheckStorageAndUnloadCargo(unload_items) --unload new cargo only
		end
		
		MfgAide.ship_cargo = {}
		MfgAide.isMisn = false
		MfgAide.isWarned = false
	end
end

function MfgAide:ENTERED_STATION(event, data)
	tryAutoload()
end

function MfgAide:PLAYER_LOGGED_OUT(event, data)
	MfgAide.isWarned = false
	
	UnregisterEvent(MfgAide, "ENTERED_STATION")
	UnregisterEvent(MfgAide, "MISSION_UPDATED")
end

MfgAide.Inputs = {stock=true,options=true,help=true} --list of valid commands
function MfgAide:input(args)
	args = args or {}
	local command = args[1] and args[1]:lower()
	if command then table.remove(args, 1) end
	
	if MfgAide.Inputs[command] then
		MfgAide[command](MfgAide, args)
	else MfgAide.stock(MfgAide) end
end

function MfgAide:stock(args) check_stock{warn_override=true} end
function MfgAide:options(args) MfgAide.ui:Show() end
function MfgAide:help(args) MfgAide.ui:ShowHelp() end

RegisterEvent(MfgAide, "PLAYER_ENTERED_GAME")
RegisterEvent(MfgAide, "MISSION_ADDED")
RegisterEvent(MfgAide, "MISSION_REMOVED")
RegisterEvent(MfgAide, "PLAYER_LOGGED_OUT")
RegisterUserCommand("mfg", MfgAide.input)